# pptap.rb version 0.2 (coding: Windows-31J)  2015/05/28
  # an ruby library for PowerPoint OLE control
  # Copyright (C) T. Yoshiizumi, 2015 All rights reserved.
require "win32ole"

# Office, PowerPoint֘A̒萔ݒp̃\bhȂ
module PptConstant
end

module MyOfficeConstant
end

def office_constant_load()
  if defined?(MyOfficeConstant::CONSTANTS)
    return nil
  end
  ms_dir = "C:/Program Files/Common Files/microsoft shared"
  nums = []
  if test(?d, ms_dir)
    Dir.glob("#{ms_dir}/OFFICE*") do |dname|
      if dname =~ /\/OFFICE(\d+)$/
        n = $1.to_i
        nums << n
      end
    end
    nums.sort
  end
  res = false
  if nums.size > 0
    office_obj = "Microsoft Office #{nums.last}.0 Object Library"
    WIN32OLE.const_load(office_obj, MyOfficeConstant)
    res = true
  end
  return res
end


# ʂɗp郁\bhQ
module Pptm
  module_function()
  def fullpath(fname)
    fname = fname.gsub(/\//, "\\")
    fso = WIN32OLE.new('Scripting.FileSystemObject')
    return fso.GetAbsolutePathName(fname)
  end

  def filename2(fname)
    fname = fullpath(fname)
    dd = File.dirname(fname)
    unless dd =~ /\\$/
      dd = dd + "\\"
    end
    ff = File.basename(fname)
    return [dd, ff]
  end

  def filename3(fname)
    fname = fullpath(fname)
    dd = File.dirname(fname)
    unless dd =~ /\\$/
      dd = dd + "\\"
    end
    ff = File.basename(fname, ".*")
    ee = File.extname(fname)
    return [dd, ff, ee]
  end
end


# Presentationŝ̂P߂̃NX
class PptPresen
  include PptConstant

  def initialize(pap)
    @app = pap
    @obj = @open_type = @filename = nil
  end
  attr_accessor :app
  attr_accessor :obj
  attr_accessor :open_type
  attr_accessor :filename

  def open(fname)
    if fname.class == String
      @filename = Pptm::fullpath(fname)
      if test(?e, @filename)
        @obj = @app.obj.Presentations.Open(@filename)
      else
        @obj = @app.obj.Presentations.Add
      end
    end
  end

  def close()
    if @obj
      da_v = @app.display_alert(false)
      @obj.Saved = true  # ۑ̂Ƃ
      @obj.Close
      @app.display_alert(da_v)
      @obj = nil
    end
  end

  def save(fname = nil, ftype = nil)
    unless @obj
      return
    end
    if fname.class != String
      fname = @filename
    else
      fname = Pptm::fullpath(fname)
    end
    err_msg = nil
    if test(?e, fname)  # ɑ݂t@C㏑
      begin
        @obj.Save
      rescue
        err_msg = $!
      end
    else  # ܂݂ĂȂt@CƂĕۑ
      unless ftype
        dd, ff, extname = Pptm::filename3(fname)
        bname = dd + ff
        ftype = @app.ppt_ext[extname]
      end
      begin
        !ftype ? @obj.SaveAs(fname) : @obj.SaveAs(bname, ftype)
      rescue
        err_msg = $!
      end
    end
    if err_msg
      err_msg = err_msg.sub(/\A\n+/, "")
      STDERR.puts "failed to save '#{fname}'!\n#{err_msg}"
    end
  end

  def add_slide(layout = PpLayoutTitleOnly)
    @obj.Slides.Add('Index'=>@obj.Slides.Count + 1,
        'Layout'=>layout)
  end

  def pdf_out(arg = {})
    hs = {'FixedFormatType'=>PpFixedFormatTypePDF}
    if arg.class == String
      @obj.PrintOptions.Ranges.ClearAll
      prng = @obj.PrintOptions.Ranges.Add(1, @obj.Slides.Count)
      arg = hs.update({'Path'=>arg, 'PrintRange'=>prng})
    elsif arg.class == Hash
      arg = hs.update(arg)
      unless arg['PrintRange']
        @obj.PrintOptions.Ranges.ClearAll
        prng = @obj.PrintOptions.Ranges.Add(1, @obj.Slides.Count)
        arg['PrintRange'] = prng
      end
    end
    if arg.class == Hash
      fname = arg['Path']
      unless fname
        fname = @filename.sub(/\.[^.]*$/, "") + ".pdf"
      end
      arg['Path'] = Pptm::fullpath(fname)
      @obj.ExportAsFixedFormat(arg)
    else
      STDERR.puts "invalid parameter of 'PptPresen::pdf_out':\n" +
          "  #{arg.inspect}"
    end
  end
  alias :pdf :pdf_out

  def method_missing(sym, *arg)
    name = sym.to_s
    if name =~ /^[A-Z_]/
      return @obj.send(sym, *arg)
    else
      STDERR.puts "undefined method `#{name}' for Pptap (NameError)"
    end
  end
end


# PowerPoint Application߂̃NX
class Pptap
  include PptConstant

  def init_set()
    @obj = nil
    begin
      @obj = WIN32OLE.new('PowerPoint.Application')
    rescue
      @obj = nil
    end
    if @obj
      @obj.Visible = true
      unless defined?(PptConstant::CONSTANTS)
        wv = $VERBOSE
        $VERBOSE = nil
        WIN32OLE.const_load(@obj, PptConstant)
        $VERBOSE = wv
      end
      if @obj.Version.to_f >= 12.0
        @ppt_ext = {'.pptx'=>PpSaveAsDefault,
            '.pptm'=>PpSaveAsOpenXMLPresentationMacroEnabled,
            '.ppsx'=>PpSaveAsOpenXMLShow,
            '.ppsm'=>PpSaveAsOpenXMLShowMacroEnabled,
            '.potx'=>PpSaveAsOpenXMLTemplate,
            '.potm'=>PpSaveAsOpenXMLTemplateMacroEnabled,
            '.odp'=>PpSaveAsOpenDocumentPresentation,
            '.ppt'=>PpSaveAsPresentation,
            '.pot'=>PpSaveAsTemplate,
            '.pps'=>PpSaveAsShow}
      else
        @ppt_ext = {'.ppt'=>PpSaveAsPresentation,
            '.pot'=>PpSaveAsTemplate,
            '.pps'=>PpSaveAsShow}
      end
    end
  end
  private :init_set
  attr_reader :obj
  attr_reader :ppt_ext

  def initialize(*fname, &block)
    ObjectSpace.define_finalizer(self, Pptap.callback)
    init_set()
    $PptapObject = @obj ? self : nil
    if @obj and fname.size > 0
      last_n = fname.size
      for n in 0...last_n
        presen = self.presen_open(fname[n])
        if block
          block.call(presen)
        end
      end
      if block
        self.quit
      end
    end
  end

  def Pptap.callback
    proc { $PptapObject.quit()  if $PptapObject != nil }
  end

  def opens(*fname, &block)
    res = []
    if @obj and fname.size > 0
      last_n = fname.size
      for n in 0...last_n
        presen = self.presen_open(fname[n])
        if block
          block.call(presen)
        else
          res << presen
        end
      end
    end
    if res.size < 1
      res = nil
    end
    return res
  end

  def close()  # ׂĂPresentations
    if @obj
      count = @obj.Presentations.Count
      if count > 0
        dav = self.display_alert(false)
        count.step(1, -1) do |n|
          @obj.Presentations.Item(n).Saved = true  # ۑ̂Ƃ
          @obj.Presentations.Item(n).Close
        end
        self.display_alert(dav)
      end
    end
  end

  def quit()
    if @obj
      self.close()
      @obj.Quit
      @obj = nil
      $PptapObject = nil
    end
  end

  def display_alert(val, &block)
    before_val = @obj.DisplayAlerts
    @obj.DisplayAlerts = val
    if block
      block.call()
      @obj.DisplayAlerts = before_val
      return
    end
    return before_val
  end

  def presen_open(fname = nil)
    presen = PptPresen.new(self)
    presen.open(fname)
    return presen
  end

  def presen_close(presen)
    if presen
      presen.close
    end
    presen.app = presen.obj = presen.open_type = presen.filename = nil
  end

  def version()  # pptap̃o[W
    return "0.2"
  end

  def method_missing(sym, *arg)
    name = sym.to_s
    if name =~ /^[A-Z_]/
      return @obj.send(sym, *arg)
    else
      STDERR.puts "undefined method `#{name}' for Pptap (NameError)"
    end
  end
end

include MyOfficeConstant
include PptConstant
if office_constant_load()
  include MyOfficeConstant
end
